/*
 *  $Id: callbacks.c,v 1.4 2003/08/01 12:05:53 luke Exp $
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <assert.h>
#include <string.h>
#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "xbr.h"
#include "rampfile.h"
#include "misc.h"

gboolean channels_combined = FALSE;

/* Helper functions */
static int curve_get_ramp(GtkCurve* curve, float ramp[]) {
	gtk_curve_get_vector(curve, ramp_size, ramp);
	return ramp_size;
}

static void set_file_selection_handlers(GtkWidget* fsel,  
			GtkSignalFunc ok_handler, GtkSignalFunc cancel_handler) {
	GtkWidget *ok_button;
	GtkWidget *cancel_button;
	
	ok_button = GTK_FILE_SELECTION (fsel)->ok_button;
	cancel_button = GTK_FILE_SELECTION (fsel)->cancel_button;
	gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      ok_handler,
                      fsel);
  	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
                      cancel_handler,
                      fsel);
}

static void clear_file_selection_handlers(GtkWidget* fsel, 
			GtkSignalFunc ok_handler, GtkSignalFunc cancel_handler) {
	GtkWidget *ok_button;
	GtkWidget *cancel_button;
	
	ok_button = GTK_FILE_SELECTION (fsel)->ok_button;
	cancel_button = GTK_FILE_SELECTION (fsel)->cancel_button;
	gtk_signal_disconnect_by_func (GTK_OBJECT (ok_button), 
					  ok_handler, fsel);
  	gtk_signal_disconnect_by_func (GTK_OBJECT (cancel_button),
                      cancel_handler, fsel);
}

/* Event handlers */

/* Main screen */

float rescale(float v, float d1, float d2) {
	if (d2 == 0 && d2 == 0)
		return 0.0;
	else 
		return v*d1/d2;
}

void get_ramps_from_curves(GtkWidget* ancor, 
	float brightness_ramp[], float red_ramp[], float green_ramp[], float blue_ramp[]) {
	curve_get_ramp(GTK_CURVE(lookup_widget(GTK_WIDGET(ancor), curve_widget_names[COMBINED])), brightness_ramp);
	curve_get_ramp(GTK_CURVE(lookup_widget(GTK_WIDGET(ancor), curve_widget_names[RED])), red_ramp);
	curve_get_ramp(GTK_CURVE(lookup_widget(GTK_WIDGET(ancor), curve_widget_names[GREEN])), green_ramp);
	curve_get_ramp(GTK_CURVE(lookup_widget(GTK_WIDGET(ancor), curve_widget_names[BLUE])), blue_ramp);
}

void
on_btn_apply_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	float brightness_ramp[MAX_RAMPSIZE];
	float red_ramp[MAX_RAMPSIZE], green_ramp[MAX_RAMPSIZE], blue_ramp[MAX_RAMPSIZE];
	int i;
	
	get_ramps_from_curves(GTK_WIDGET(button), brightness_ramp, red_ramp, green_ramp, blue_ramp);
	
	for (i = 0; i < ramp_size; ++i) {
		red_ramp[i] = MIN(1.0, rescale(red_ramp[i], brightness_ramp[i], reference_ramp[i]));
		green_ramp[i] = MIN(1.0, rescale(green_ramp[i], brightness_ramp[i], reference_ramp[i]));
		blue_ramp[i] = MIN(1.0, rescale(blue_ramp[i], brightness_ramp[i], reference_ramp[i]));
	}
	set_brightness(ramp_size, red_ramp, green_ramp, blue_ramp);
}

GtkCurve* get_current_curve(GtkWidget* ancor) {
	GtkWidget* notebook;
	gint channel_num;
	notebook = lookup_widget(GTK_WIDGET(ancor), "notebook_curves");
	channel_num = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	return GTK_CURVE(lookup_widget(GTK_WIDGET(notebook), curve_widget_names[channel_num]));
}

void
on_btn_reset_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCurve* curve;
	curve = get_current_curve(GTK_WIDGET(button));	
	gtk_curve_reset(curve);
}

void
on_btn_save_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget* fsel;
	
	fsel = lookup_widget(GTK_WIDGET(user_data), "fsel_save");
	set_file_selection_handlers(fsel, 
			GTK_SIGNAL_FUNC(on_ok_button_save_clicked), GTK_SIGNAL_FUNC(on_cancel_button_save_clicked));	
	gtk_widget_show(fsel);	
}

void
on_btn_load_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget* fsel;
	
	fsel = lookup_widget(GTK_WIDGET(user_data), "fsel_save");
	set_file_selection_handlers(fsel, 
			GTK_SIGNAL_FUNC(on_ok_button_load_clicked), GTK_SIGNAL_FUNC(on_cancel_button_load_clicked));	
	gtk_widget_show(fsel);
}

/* File dialog */

void
on_ok_button_save_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	float brightness_ramp[MAX_RAMPSIZE];
	float red_ramp[MAX_RAMPSIZE], green_ramp[MAX_RAMPSIZE], blue_ramp[MAX_RAMPSIZE];
	GtkWidget *curve, *fsel, *window_main;
	gchar* fname;
	int res;
	
	fsel = GTK_WIDGET(user_data);

	gtk_widget_hide(GTK_WIDGET(fsel));
	clear_file_selection_handlers(fsel, 
			GTK_SIGNAL_FUNC(on_ok_button_save_clicked), GTK_SIGNAL_FUNC(on_cancel_button_save_clicked));	
	
	window_main = lookup_widget(GTK_WIDGET(fsel), "window_main");
	curve = lookup_widget(window_main, "curve_combined");
	
	get_ramps_from_curves(window_main, brightness_ramp, red_ramp, green_ramp, blue_ramp);
	fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fsel));
	res = save_ramps_to_file(fname, ramp_size, brightness_ramp, red_ramp, green_ramp, blue_ramp);	
	if (res != 0) 
		show_msg(strerror(errno));
	else
		show_msg("Profile saved successfully.");
}


void
on_cancel_button_save_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fsel;
	fsel = GTK_WIDGET(user_data);
	gtk_widget_hide(GTK_WIDGET(fsel));
	
	clear_file_selection_handlers(fsel, 
			GTK_SIGNAL_FUNC(on_ok_button_save_clicked), GTK_SIGNAL_FUNC(on_cancel_button_save_clicked));	
}


void
on_ok_button_load_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	int size;
	float brightness_ramp[MAX_RAMPSIZE];
	float red_ramp[MAX_RAMPSIZE], green_ramp[MAX_RAMPSIZE], blue_ramp[MAX_RAMPSIZE];
	GtkWidget *fsel, *window_main;
	gchar* fname;
	
	fsel = GTK_WIDGET(user_data);

	gtk_widget_hide(GTK_WIDGET(fsel));
	clear_file_selection_handlers(fsel, 
			GTK_SIGNAL_FUNC(on_ok_button_load_clicked), GTK_SIGNAL_FUNC(on_cancel_button_load_clicked));	
	
	window_main = lookup_widget(GTK_WIDGET(fsel), "window_main");

	fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fsel));
	
	size = read_ramps_from_file(fname, brightness_ramp, red_ramp, green_ramp, blue_ramp);
	switch (size) {
		case -3:
			show_msg("Wrong ramp size in file.");
			return;
		case -2:
			show_msg("Ramp value out of range.");
			return;
		case -1:
			show_msg(strerror(errno));
			return;
	}
	if (size < 0)
		return;

	update_curves_from_ramps(window_main, brightness_ramp, red_ramp, green_ramp, blue_ramp);
	show_msg("Profile loaded successfully.");
}


void
on_cancel_button_load_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fsel;
	fsel = GTK_WIDGET(user_data);
	gtk_widget_hide(GTK_WIDGET(fsel));
	
	clear_file_selection_handlers(fsel, 
			GTK_SIGNAL_FUNC(on_ok_button_load_clicked), GTK_SIGNAL_FUNC(on_cancel_button_load_clicked));	
}

/*
void
on_checkbutton_combine_colors_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	int i;
	GtkWidget* window_main, *notebook;
	window_main = GTK_WIDGET(user_data);
	notebook = lookup_widget(window_main, "notebook_curves");
	channels_combined = gtk_toggle_button_get_active(togglebutton);

	if (channels_combined == TRUE) {
		for (i = RED; i <= BLUE; ++i) {
			gtk_notebook_remove_page (GTK_NOTEBOOK(notebook), 1);
		}
	} else {
		for (i = RED; i <= BLUE; ++i) {
			gtk_notebook_insert_page(GTK_NOTEBOOK(notebook), 
				lookup_widget(window_main, channel_name[i]), gtk_label_new(channel_name[i]), i);
		}
	}
}
*/

void
on_btn_reset_all_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCurve* curve;
	int i;
	for (i = COMBINED; i <= BLUE; ++i) {
		curve = GTK_CURVE(lookup_widget(GTK_WIDGET(button), curve_widget_names[i]));	
		gtk_curve_reset(curve);
	}
}
