/*
 *  $Id: main.c,v 1.4 2003/08/01 12:05:53 luke Exp $
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <getopt.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "interface.h"
#include "support.h"
#include "xbr.h"
#include "rampfile.h"
#include "misc.h"

/* work around for a strange gtk+ bug
	GtkCurve, that have never been visible, are messed up after the 
	first gtk_curve_set_vector_call().
*/
void fixbug(GtkWidget* window_main) {
	int i;
	for (i = COMBINED; i <= BLUE; ++i) {
		GtkCurve* curve;
		curve = GTK_CURVE(lookup_widget(window_main, curve_widget_names[i]));
		curve_update_from_ramp(curve, ramp_size, reference_ramp);
		curve_update_from_ramp(curve, ramp_size, reference_ramp);
		gtk_curve_reset(curve);
	}
}

void usage() {
	printf("Usage: xbrightness-gui [-h] [-r] [-p file]\n\n");
	printf(" -h        Shows this help message.\n");
	printf(" -r        Reset settings to defaults.\n");
	printf(" -p file   If this option is present, the brightness profile\n");
	printf("           store in \"file\" will be loaded. No GUI will shown.\n");
	printf("\n");
}

static int statusbar_handler_id;

static gint statusbar_handler(GtkStatusbar* sb) {
	gtk_statusbar_pop(sb, gtk_statusbar_get_context_id(sb, "generic"));
	return TRUE;
}

int main (int argc, char *argv[])
{
	GtkWidget *window_main, *fsel;
  	int c, i;
	char profile_fn[256];
	gboolean use_profile = FALSE, use_default_profile = FALSE;
		
	gtk_set_locale ();
	gtk_init (&argc, &argv);

	add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
	add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

 	while (1) {
		c = getopt(argc, argv, "hrp:");
		if (c == -1) 
			break;
		
		switch (c) {
			case 'p':
				strncpy(profile_fn, optarg, sizeof(profile_fn));
				assert(profile_fn[sizeof(profile_fn)-1] == '\0');
				use_profile = TRUE;
				break;
			case 'r':
				use_default_profile = TRUE;
				break;
			case 'h':
				usage();
				return 0;
				break;
			case '?':
				usage();
				return 2;
				break;
		}
	
	}

	ramp_size = get_ramp_size();
	for (i = 0; i < ramp_size; ++i)
		reference_ramp[i] = 1.0*i/ramp_size;
	
	// reset everything
	if (use_default_profile) {
		set_brightness(ramp_size, reference_ramp, reference_ramp, reference_ramp);
		return 0;
	}
	
	if (use_profile) {
		int ramp_size_file;
		float ramp[MAX_RAMPSIZE];
		
		assert(ramp_size > 0);
		
		ramp_size_file = read_ramp(profile_fn, ramp);
		assert(ramp_size_file == ramp_size);
		
		set_brightness(ramp_size, ramp, ramp, ramp);
		
		return 0;
	}

	window_main = create_window_main ();
	fsel = create_fsel_save ();
	gtk_object_set_data (GTK_OBJECT (window_main), "fsel_save", fsel);	
	
	for (i = RED; i <= BLUE; ++i) {
		GtkWidget* notebook = lookup_widget(window_main, "notebook_curves");
		GtkWidget* child;
		child = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), i);
		gtk_object_set_data (GTK_OBJECT (window_main), channel_name[i], child);	
	}
	
	gtk_object_set_data (GTK_OBJECT (fsel), "window_main", window_main);	
	
	gtk_widget_show (window_main);
	
	fixbug(window_main);
	
	main_statusbar = GTK_STATUSBAR(lookup_widget(window_main, "statusbar_main"));
	curve_update_from_current(window_main);
	
	statusbar_handler_id = gtk_timeout_add(3000, (GtkFunction)statusbar_handler, GTK_WIDGET(main_statusbar));
	gtk_main ();
	return 0;
}
