/*
 *  $Id: misc.c,v 1.4 2003/08/01 12:05:53 luke Exp $
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <assert.h>
#include "support.h"
#include "xbr.h"
#include "misc.h"

char* channel_name[]  = {"combined", "red", "green", "blue"};
char* curve_widget_names[] = {"curve_combined", "curve_red", "curve_green", "curve_blue"};
float reference_ramp[MAX_RAMPSIZE];
int ramp_size;
GtkStatusbar* main_statusbar;

void update_curves_from_ramps(GtkWidget* ancor, float brightness_ramp[], float red_ramp[], float green_ramp[], float blue_ramp[]) {
	curve_update_from_ramp(GTK_CURVE(lookup_widget(ancor, "curve_combined")), ramp_size, brightness_ramp);
	curve_update_from_ramp(GTK_CURVE(lookup_widget(ancor, "curve_red")), ramp_size, red_ramp);
	curve_update_from_ramp(GTK_CURVE(lookup_widget(ancor, "curve_green")), ramp_size, green_ramp);
	curve_update_from_ramp(GTK_CURVE(lookup_widget(ancor, "curve_blue")), ramp_size, blue_ramp);
}

void curve_update_from_ramp (GtkCurve* curve, int size, float ramp[]) {
	gtk_curve_set_vector(GTK_CURVE(curve), size, ramp);	
	gtk_curve_set_curve_type(GTK_CURVE(curve), GTK_CURVE_TYPE_SPLINE);
}

void curve_update_from_current (GtkWidget* ancor) {
	float ramps[4][MAX_RAMPSIZE];
	gboolean all_equal = TRUE;	
	int i;
	GtkWidget *curves[4];
	
	get_brightness(ramps[RED], ramps[GREEN], ramps[BLUE]);
	
	for (i = 0; i < ramp_size; ++i) {
		if (!(ramps[RED][i] == ramps[GREEN][i] && ramps[GREEN][i] == ramps[BLUE][i])) {
			all_equal = FALSE;
			break;
		}
	}
	
	for (i = COMBINED; i <= BLUE; ++i) {
		curves[i] = lookup_widget(ancor, curve_widget_names[i]);
		gtk_curve_reset(GTK_CURVE(curves[i]));
	}
	
	if (all_equal) {
		/* use any, as all are equal */
		curve_update_from_ramp(GTK_CURVE(curves[COMBINED]), ramp_size, ramps[RED]);
	} else {
		curve_update_from_ramp(GTK_CURVE(curves[RED]), ramp_size, ramps[RED]);
		curve_update_from_ramp(GTK_CURVE(curves[GREEN]), ramp_size, ramps[GREEN]);
		curve_update_from_ramp(GTK_CURVE(curves[BLUE]), ramp_size, ramps[BLUE]);
	}
}

void show_msg(char* text) {
	gtk_statusbar_push(main_statusbar, gtk_statusbar_get_context_id(main_statusbar, "generic"), text);
}
