/*
 *  $Id: xbr.c,v 1.4 2003/08/01 12:05:53 luke Exp $
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
#include <assert.h>
#include <stdio.h>

#include "xbr.h"

static int connect_to_display (Display** display, int* screen) {
	*display = XOpenDisplay(NULL); /* connect do default display */
	
	if ((*display) == NULL)
		return -1;
	
	*screen = XDefaultScreen(*display);
	
	return 0;
}

/* returns non-zero on failure */
int set_brightness(int size, float red_ramp[], float green_ramp[], float blue_ramp[]) {
	Display* display;
	int screen, i;
	unsigned short r_s_ramp[MAX_RAMPSIZE], g_s_ramp[MAX_RAMPSIZE], b_s_ramp[MAX_RAMPSIZE];
	
	if (connect_to_display(&display, &screen))
		return -1;
	
	for (i = 0; i < size; ++i) {
		r_s_ramp[i] = (unsigned short)(red_ramp[i]*MAX_BRIGHTNESS);
		g_s_ramp[i] = (unsigned short)(green_ramp[i]*MAX_BRIGHTNESS);
		b_s_ramp[i] = (unsigned short)(blue_ramp[i]*MAX_BRIGHTNESS);
	}
	
	if (!XF86VidModeSetGammaRamp(display, screen, size, r_s_ramp, g_s_ramp, b_s_ramp)) {
		XCloseDisplay(display);
		return -1;
	}
	
	XCloseDisplay(display);
	
	return 0;
}

/* returns -1 on failure, otherwise the size of the ramps */
int get_brightness(float red_ramp[], float green_ramp[], float blue_ramp[]) {
	Display* display;
	int screen, size, i;
	unsigned short red_short_ramp[MAX_RAMPSIZE], green_short_ramp[MAX_RAMPSIZE], blue_short_ramp[MAX_RAMPSIZE];
	
	if (connect_to_display(&display, &screen))
		return -1;
	
	size = get_ramp_size();
	if (size == -1) 
		return -1;
	
	if (!XF86VidModeGetGammaRamp(display, screen, size, red_short_ramp, green_short_ramp, blue_short_ramp))
		size = -1; /* failure */
	else	
		for (i = 0; i < size; ++i) {  /* normal case */
			red_ramp[i] = (float)red_short_ramp[i]/MAX_BRIGHTNESS;
			green_ramp[i] = (float)green_short_ramp[i]/MAX_BRIGHTNESS;
			blue_ramp[i] = (float)blue_short_ramp[i]/MAX_BRIGHTNESS;
		}
	
	XCloseDisplay(display);
	
	return size;
}

/* returns -1 on failure */
int get_ramp_size() {
	Display *display;
	int screen, size;
	
	if (connect_to_display(&display, &screen))
		return -1;
		
	if (!XF86VidModeGetGammaRampSize(display, screen, &size))
		size = -1;
	
	XCloseDisplay(display);

	return size;
}
